"""
The edition manual is a real concept translated into programming.

Think about a content creator that always edits his videos in the
same way (him or his paid editor). The editor has a bank of memes,
sounds, images, and tries to do the same in all the videos he 
edits. This is using your edition guideline. Applying your edition
manual. And that's the idea we want to build here.

When you are editing, you put some images when some concept is 
mentioned, or maybe a sound is played when you say a joke. Or,
simply, a censorship sound is added when a bad word is said.

All this can be automated by writting this edition manual and
telling the software how and when you want the clip to be edited
with any resource you want. You maybe want an effect, a sound, a
sticker image, some specific video clip played overlay during X
seconds. All this is what you can do here.
"""